package com.amphinicy.mhp.communication.client;


import java.util.Vector;
import java.util.Enumeration;
import com.amphinicy.mhp.util.ThreadRunner;

public abstract class Controller
    implements IController
{
  /**
   * Helper class for getting new messages from server
   */
  protected class ConnectionRunner
      extends ThreadRunner {
    Controller controller;

    public ConnectionRunner(Controller controller) {
      this.controller = controller;
    }

    public String getName() {
      return controller.getClass().getName();
    }

    public void run() {
      // Receive new messages from server
      controller.getNextMessages();
    }
  }


  /**
   * Status of Controller
   */
  private boolean connected = false;

  /**
   * Message listeners
   */
  protected Vector messageListeners = new Vector();

  /**
   * Timeout for sending messages
   */
  protected long timeout = 10000;

  public void connect()
      throws ControllerException
  {
    connected = true;
  }

  public void disconnect()
      throws ControllerException
  {
    connected = false;
  }

  public boolean isConnected()
  {
    return connected;
  }

  synchronized public void addMessageListener(MessageListener l)
  {
    messageListeners.addElement(l);
  }

  synchronized public void removeMessageListener(MessageListener l)
  {
    messageListeners.removeElement(l);
  }

  synchronized public void clearMessageListener()
  {
    messageListeners.removeAllElements();
  }

  /**
   * Send to listeners new message.
   * @param message ControllerMessage
   */
  synchronized protected void notifyMessageListener(ControllerMessage message)
  {
    Enumeration e = messageListeners.elements();
    while(e.hasMoreElements())
      ((MessageListener)e.nextElement()).messageArrived(new MessageEvent(message));
  }

  /**
   *
   * @param timeout long
   */
  public void setTimeout(long timeout) {
    this.timeout = timeout;
  }

  /**
   *
   * @return long
   */
  public long getTimeout() {
    return timeout;
  }

}
