package com.amphinicy.mhp.communication.client;

import java.net.DatagramSocket;
import java.net.DatagramPacket;
import java.net.InetAddress;
import java.io.*;
import java.net.*;
import com.amphinicy.mhp.util.Utils;


public class DatagramSocketController
    extends Controller {
  private InetAddress address;
  private int port;
  private int clientPort;
  private DatagramSocket s = null;
  private ConnectionRunner runner;

  private ControllerMessage sendMessage = null;

  /**
   * Constructor.
   * @param address String address of server
   * @param clientPort int client port
   * @param serverPort int server port
   * @throws ControllerException
   */
  public DatagramSocketController(String address, int clientPort, int serverPort)
      throws ControllerException
  {
    try {
      this.address = InetAddress.getByName(address);
    }
    catch (UnknownHostException ex) {
       throw new ControllerException(ex.getMessage());
    }
    this.port = serverPort;
    this.clientPort = clientPort;
    runner = new ConnectionRunner(this);
  }

  public synchronized void connect() throws ControllerException {
    try {
      // Create datagram socket on port clientPort
      s = new DatagramSocket(this.clientPort);
      try {
        // Set timeout for receiving data
        s.setSoTimeout((int)timeout);
      }
      catch (Exception ex1) {
      }
    }
    catch (Exception ex) {
      throw new ControllerException(ex.getMessage());
    }
    // Enable controller and start getting messages from server
    super.connect();
    runner.start();
  }

  public synchronized void disconnect() throws ControllerException {

    // Disable controller and stop getting messages from server
    super.disconnect();

    runner.stop();

    if (s != null) {
      try {
        // close datagram socket
        s.close();
      }
      catch (Exception ex) {
        throw new ControllerException(ex.getMessage());
      }
    }
  }

  public synchronized void post(ControllerMessage message) throws
      ControllerException {
    // set type to post
    message.setPayloadType(message.POST);
    byte[] data = message.getMessageData();
    try {
      // send message to server address:port
      s.send(new DatagramPacket(data, data.length, address, port));
    }
    catch (IOException ex) {
      throw new ControllerException(ex.getMessage());
    }
  }

  public synchronized ControllerMessage send(ControllerMessage message) throws
      ControllerException {
    ControllerMessage returnMessage = null;
    // set type to send
    message.setPayloadType(message.SEND);
    byte[] data = message.getMessageData();
    try {
      // send message to server address:port
      s.send(new DatagramPacket(data, data.length, address, port));
    }
    catch (IOException ex) {
      throw new ControllerException(ex.getMessage());
    }

    synchronized (this.s){
      try {
        // wait timeout
        s.wait(timeout);
        returnMessage = this.sendMessage;
      }
      catch (InterruptedException ex1) {
      }
    }
    return returnMessage;
  }

  public void getNextMessages() {
    byte receive[] = new byte[65507];
    DatagramPacket receivePacket = new DatagramPacket(receive, receive.length);
    while (isConnected()) {
      String data = null;
      receivePacket.setLength(receive.length);
      try {
        // Receive new message
        s.receive(receivePacket);
        if (receivePacket.getLength() > 0) {
          synchronized (this.s){
            ControllerMessage message = new ControllerMessage();
            message.setMessageData(receivePacket.getData());
            System.out.println("Received size: " + receivePacket.getLength());
            System.out.println("Received: " + Utils.byteToHex(receivePacket.getData(), receivePacket.getLength()));
            if (message.getType() == message.POST) {
              notifyMessageListener(message);
            }
            else {
              this.sendMessage = message;
              s.notify();
            }
          }
        }
      }
      catch (IOException ex) {
        ex.printStackTrace();
      }
    }
  }
}
