package com.amphinicy.mhp.communication.client;

import java.net.Socket;
import java.net.*;
import java.io.*;
import java.io.InterruptedIOException;

public class SocketController
    extends Controller {
  private String address;
  private int port;
  private Socket s = null;
  private ConnectionRunner runner;
  private DataOutputStream out = null;
  private DataInputStream in = null;

  private ControllerMessage sendMessage = null;

  /**
   * Connect to address:port server
   * @param address String
   * @param port int
   */
  public SocketController(String address, int port) {
    this.address = address;
    this.port = port;
    runner = new ConnectionRunner(this);
  }

  public synchronized void connect() throws ControllerException {
    try {
      s = new Socket(address, port);
      try {
        s.setSoTimeout((int)timeout);
      }
      catch (Exception ex1) {
      }

      // Create streams for reading and writing lines of text
      // from and to this socket.
      out = new DataOutputStream(s.getOutputStream());
      in = new DataInputStream(s.getInputStream());
    }
    catch (Exception ex) {
      throw new ControllerException(ex.getMessage());
    }
    super.connect();
    runner.start();
  }

  public synchronized void disconnect() throws ControllerException {
    super.disconnect();

    runner.stop();

    if (s != null) {
      try {
        out.close();
        in.close();
        s.close();
      }
      catch (Exception ex) {
        throw new ControllerException(ex.getMessage());
      }
    }
  }

  public synchronized void post(ControllerMessage message) throws
      ControllerException {
    try {
      message.setPayloadType(message.POST);
      out.write(message.getMessageData());
    }
    catch (IOException ex) {
      throw new ControllerException(ex.getMessage());
    }
  }

  public synchronized ControllerMessage send(ControllerMessage message) throws
      ControllerException {
    ControllerMessage returnMessage = null;
    try {
      message.setPayloadType(message.SEND);
      out.write(message.getMessageData());
    }
    catch (IOException ex) {
      throw new ControllerException(ex.getMessage());
    }

    synchronized (this.in){
      try {
        in.wait(timeout);
        returnMessage = this.sendMessage;
      }
      catch (InterruptedException ex1) {
      }
    }
    return returnMessage;
  }

  public void getNextMessages() {

    while (isConnected()) {
      String data = null;
      try {
        ControllerMessage message = new ControllerMessage(in);
        if (message.getType() > -1) {
          synchronized (this.in){
            if (message.getType() == message.POST) {
              notifyMessageListener(message);
            }
            else {
              this.sendMessage = message;
              this.in.notify();
            }
          }
        }
      }
      catch (IOException ex) {
        if(ex instanceof InterruptedIOException)
          System.out.println(ex.getMessage());
        else
        {
          ex.printStackTrace();
          break;
        }
      }
    }
  }

}
